package org.verohallinto.apitamoclient.util;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Properties-tiedoston staattiset ksittelymetodit.
 */
public class Props {

    private static Logger log = LoggerFactory.getLogger(Props.class);
    private static final String APP_VERSION = "app.version";
    private static final String APP_WSDL = "app.wsdl";
    private static final String APP_TIMEOUT_CONN = "app.connection.timeout.ms";
    private static final String APP_TIMEOUT_READ = "app.read.timeout.ms";
    private static boolean isSet = false;
    private static String env;
    private static Properties props;
    private static Map<String, String> emails;

    /**
     * @param env <code>String</code> environment to use.
     */
    public static void init(final String env) {

        if (log.isDebugEnabled()) {
            log.debug("init() - begin. env=" + env);
        }

        if ("dev".equals(env) || "devtest".equals(env) || "test".equals(env) || "prod".equals(env)) {
            Props.env = env.concat(".");

            if (!isSet) {
                loadProperties("wsapp-pkiclient.properties");
                setEmails();
                isSet = true;
            }
        } else {
            throw new IllegalArgumentException("init() - Illegal argument: " + env);
        }

        if (log.isDebugEnabled()) {
            log.debug("init() - end.");
        }
    }

    /**
     * Annetun avaimen mukaisen properties-tiedon palautus.
     * </p>
     *
     * @param key <code>String</code> the key of the property.
     * @return Avaimen arvo tai null, jos avainta ei lydy.
     */
    public static String get(String key) {

        if (log.isDebugEnabled()) {
            log.debug("get() - begin. Key=" + env + key);
        }

        key = env.concat(key);
        String value;

        if (props.containsKey(key)) {
            value = props.getProperty(key);
        } else {
            value = null;
            log.error("get() - Key not found: " + key);
        }

        if (log.isDebugEnabled()) {
            log.debug("get() - end. Value=" + value);
        }

        return value;
    }

    /**
     * Annetun avaimen mukaisen properties-tiedon int-arvon palautus.
     * </p>
     *
     * @param key <code>String</code> key of the property.
     * @param def <code>int</code> default value if property was not found.
     * @return Avaimen int-arvo tai annettu oletusarvo jos avainta ei lydy.
     */
    public static int getInt(final String key, final int def) {

        int result;
        final String value = get(key);

        if (Utils.isNumeric(value)) {
            result = Integer.parseInt(value);
        } else {
            result = def;
        }

        return result;
    }

    /**
     * Annetun avaimen mukaisen properties-tiedon int-arvon palautus.
     * </p>
     *
     * @param key <code>String</code> the key of the property.
     * @param def <code>int</code> default value if property was not found.
     * @return Avaimen int-arvo tai annettu oletusarvo jos avainta ei lydy.
     */
    public static long getLong(final String key, final int def) {

        long result;
        final String value = get(key);

        if (Utils.isNumeric(value)) {
            result = Long.parseLong(value);
        } else {
            result = def;
        }

        return result;
    }

    public static String getVersion() {

        String value;

        if (props.containsKey(APP_VERSION)) {
            value = props.getProperty(APP_VERSION);
        } else {
            value = null;
            log.error("getVersion() - Key not found: " + APP_VERSION);
        }

        return value;
    }

    public static String getWsdl() {

        String value;

        if (props.containsKey(APP_WSDL)) {
            value = props.getProperty(APP_WSDL);
        } else {
            value = null;
            log.error("getWsdl() - Key not found: " + APP_WSDL);
        }

        return value;
    }

    public static long getTimeoutConn() {

        long value;

        if (props.containsKey(APP_TIMEOUT_CONN)) {
            value = Long.parseLong(props.getProperty(APP_TIMEOUT_CONN));
        } else {
            value = 1000 * 30;
            log.error("getTimeoutConn() - Key not found: " + APP_TIMEOUT_CONN);
        }

        return value;
    }

    public static long getTimeoutRead() {

        long value;

        if (props.containsKey(APP_TIMEOUT_READ)) {
            value = Long.parseLong(props.getProperty(APP_TIMEOUT_READ));
        } else {
            value = 1000 * 300;
            log.error("getTimeoutRead() - Key not found: " + APP_TIMEOUT_READ);
        }

        return value;
    }

    /**
     * Loads properties from a file.
     * </p>
     *
     * @param file <code>String</code> file to load.
     */
    private static void loadProperties(final String file) {

        if (log.isDebugEnabled()) {
            log.debug("loadProperties() - begin. File=" + file);
        }

        BufferedInputStream bis = null;
        props = new Properties();

        try {
            if (!file.contains("\\") && !file.contains("/")) {
                // File from classpath
                bis = new BufferedInputStream(Props.class.getClassLoader().getResourceAsStream(file));
            } else {
                // File from given path
                bis = new BufferedInputStream(new FileInputStream(file));
            }

            props.load(bis);
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException ignore) {
                }
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("loadProperties() - end. size=" + props.size());
        }
    }

    public static String getEnv() {
        return env;
    }

    public static String getEmail(final String username) {

        if (emails.containsKey(username)) {
            return emails.get(username);
        } else {
            log.warn("getEmail() - User not found: '" + username + "'");
        }

        return "";
    }

    private static void setEmails() {

        emails = new HashMap<>();
        final String value = props.getProperty("app.email.addresses");

        if (!Utils.isEmpty(value)) {
            String[] values = value.split("\\|");

            if (values.length > 0) {
                for (String s : values) {
                    int x = s.indexOf('=');

                    if (x != -1) {
                        emails.put(s.substring(0, x), s.substring(x + 1));
                    }
                }
            }
        }
    }
}
